import six

from ipalib import api, errors
from ipalib import Bool, Str, errors, Int, Flag
from ipalib.parameters import Dict
from ipalib import output
from ipalib.plugable import Registry
from ipalib.util import validate_domain_name, normalize_hostname
from ipaserver.plugins.baseldap import (
    LDAPObject,
    LDAPQuery,
    LDAPCreate,
    LDAPRetrieve,
    LDAPDelete,
    LDAPUpdate,
    LDAPSearch,
    LDAPAddMember,
    LDAPRemoveMember,
    LDAPAddAttribute,
    LDAPModAttribute,
    LDAPRemoveAttribute,
    add_missing_object_class,
    pkey_to_value)
from ipalib import _, ngettext
from ipapython.dn import DN
from ipaserver.plugins.group import group, group_add, group_mod
from ipaserver.plugins.host import host, host_add
from ipaserver.plugins.user import user, user_add, user_mod, user_show
from ipaserver.plugins.machines import machine, machine_show
from ipaserver.plugins.machinerole import machinerole, machinerole_show
from ipaserver.plugins.machinegroup import machinegroup, machinegroup_show
from ipaserver.plugins.network import network, network_show
from ipaserver.plugins.networkinterface import networkinterface, networkinterface_show

if six.PY3:
    unicode = str

__doc__ = _("""
Configuration object
""") + _("""
Set configuration key value data to network, network interface, 
machine, machine group or machine role.

EXAMPLES:
 Set machine config:
   ipa machine-config-set somemachine somekey somevalue

 Unset machine config:
   ipa machine-config-unset somemachine somekey

 Set machine group config:
   ipa machinegroup-config-set somemachine somekey somevalue

 Unset machine group config:
   ipa machinegroup-config-unset somemachine somekey

 Set machine role config:
   ipa machinerole-config-set somemachine somekey somevalue

 Unset machine role config:
   ipa machinerole-config-unset somemachine somekey

""")

register = Registry()

configobject_show_params = (
    Str('imsconfig*',
        cli_name='config',
        label=_('Config'),
        flags={'no_create', 'no_update', 'no_search', 'no_option', 'no_display' },
    ),
    Dict('configuration?',
        cli_name='configuration',
        label=_('Configuration data'),
#        flags={'virtual_attribute', 'no_create', 'no_update', 'no_search'},
        flags={'virtual_attribute'},
    ),
)

def configobject_show_post_callback(self, ldap, dn, entry_attrs, *keys, **options):
    if 'imsconfig' in entry_attrs:
        config = dict(s.split('=') for s in list(entry_attrs.get('imsconfig', [])))
        entry_attrs['configuration'] = config
        entry_attrs.pop('imsconfig')
    return dn

machine.managed_permissions = {**machine.managed_permissions, **{
    'System: Read Machine Config Attributes': {
        'ipapermbindruletype': 'all',
        'ipapermright': {'read', 'search', 'compare'},
        'ipapermdefaultattr': {'imsConfig'}
    },
    'System: Modify Machine Config Attributes': {
        'ipapermbindruletype': 'permission',
        'ipapermright': {'write', 'add', 'delete'},
        'ipapermdefaultattr': {'imsConfig'}
    }
}}
machine.default_attributes = machine.default_attributes + ['imsConfig']
machine.takes_params += configobject_show_params
machine_show.register_post_callback(configobject_show_post_callback)

machinegroup.managed_permissions = {**machinegroup.managed_permissions, **{
    'System: Read Machine Group Config Attributes': {
        'ipapermbindruletype': 'all',
        'ipapermright': {'read', 'search', 'compare'},
        'ipapermdefaultattr': {'imsConfig'}
    },
    'System: Modify Machine Group Config Attributes': {
        'ipapermbindruletype': 'permission',
        'ipapermright': {'write', 'add', 'delete'},
        'ipapermdefaultattr': {'imsConfig'}
    }
}}
machinegroup.default_attributes = machinegroup.default_attributes + ['imsConfig', 'configuration']
machinegroup.takes_params += configobject_show_params
machinegroup_show.register_post_callback(configobject_show_post_callback)

machinerole.managed_permissions = {**machinerole.managed_permissions, **{
    'System: Read Machine Role Config Attributes': {
        'ipapermbindruletype': 'all',
        'ipapermright': {'read', 'search', 'compare'},
        'ipapermdefaultattr': {'imsConfig'}
    },
    'System: Modify Machine Role Config Attributes': {
        'ipapermbindruletype': 'permission',
        'ipapermright': {'write', 'add', 'delete'},
        'ipapermdefaultattr': {'imsConfig'}
    }
}}
machinerole.default_attributes = machinerole.default_attributes + ['imsConfig', 'configuration']
machinerole.takes_params += configobject_show_params
machinerole_show.register_post_callback(configobject_show_post_callback)


network.managed_permissions = {**network.managed_permissions, **{
    'System: Read Network Config Attributes': {
        'ipapermbindruletype': 'all',
        'ipapermright': {'read', 'search', 'compare'},
        'ipapermdefaultattr': {'imsConfig'}
    },
    'System: Modify Network Config Attributes': {
        'ipapermbindruletype': 'permission',
        'ipapermright': {'write', 'add', 'delete'},
        'ipapermdefaultattr': {'imsConfig'}
    }
}}
network.default_attributes = network.default_attributes + ['imsConfig', 'configuration']
network.takes_params += configobject_show_params
network_show.register_post_callback(configobject_show_post_callback)

networkinterface.managed_permissions = {**network.managed_permissions, **{
    'System: Read Network Interface Config Attributes': {
        'ipapermbindruletype': 'all',
        'ipapermright': {'read', 'search', 'compare'},
        'ipapermdefaultattr': {'imsConfig'}
    },
    'System: Modify Network Interface Config Attributes': {
        'ipapermbindruletype': 'permission',
        'ipapermright': {'write', 'add', 'delete'},
        'ipapermdefaultattr': {'imsConfig'}
    }
}}
networkinterface.default_attributes = networkinterface.default_attributes + ['imsConfig', 'configuration']
networkinterface.takes_params += configobject_show_params
networkinterface_show.register_post_callback(configobject_show_post_callback)

