import six

import logging
from ipalib import api, errors
from ipalib import Bool, Str, errors, Int, Flag
from ipalib.parameters import Dict
from ipalib import output
from ipalib.plugable import Registry
from ipalib.util import validate_domain_name, normalize_hostname
from ipaserver.plugins.baseldap import (
    LDAPObject,
    LDAPQuery,
    LDAPCreate,
    LDAPRetrieve,
    LDAPDelete,
    LDAPUpdate,
    LDAPSearch,
    LDAPAddMember,
    LDAPRemoveMember,
    LDAPAddAttribute,
    LDAPModAttribute,
    LDAPRemoveAttribute,
    add_missing_object_class,
    pkey_to_value)
from ipalib import _, ngettext
from ipapython.dn import DN
from ipaserver.plugins.group import group, group_add, group_mod
from ipaserver.plugins.host import host, host_add
from ipaserver.plugins.user import user, user_add, user_mod, user_show
from ipaserver.plugins.machines import machine, machine_show
from ipaserver.plugins.machinerole import machinerole, machinerole_show
from ipaserver.plugins.machinegroup import machinegroup, machinegroup_show
from ipaserver.plugins.network import network, network_show
from ipaserver.plugins.networkinterface import networkinterface, networkinterface_show

if six.PY3:
    unicode = str

__doc__ = _("""
Configuration object
""") + _("""
Set configuration key value data to network, network interface, 
machine, machine group or machine role.

EXAMPLES:
 Set machine config:
   ipa machine-config-set somemachine somekey somevalue

 Unset machine config:
   ipa machine-config-unset somemachine somekey

 Set machine group config:
   ipa machinegroup-config-set somemachine somekey somevalue

 Unset machine group config:
   ipa machinegroup-config-unset somemachine somekey

 Set machine role config:
   ipa machinerole-config-set somemachine somekey somevalue

 Unset machine role config:
   ipa machinerole-config-unset somemachine somekey

""")

logger = logging.getLogger(__name__)

register = Registry()

class LDAPConfigSet(LDAPQuery):
    __doc__ = _('Add new config to entry')

    msg_summary = _('Added new config to "%(value)s"')

    has_output = LDAPQuery.has_output

    takes_params = (
        Str('key',
            cli_name='key',
            label=_('Config key'),
            flags={'virtual_attribute', 'no_create', 'no_update', 'no_search'},
            primary_key=True,
        ),
        Str('value',
            cli_name='value',
            label=_('Config value'),
            flags={'virtual_attribute', 'no_create', 'no_update', 'no_search'},
            primary_key=True,
        ),
    )
    def get_args(self):
        for key in self.obj.get_ancestor_primary_keys():
            yield key
        for arg in super(LDAPConfigSet, self).get_args():
            logger.debug("LDAPConfigSet: Argument  %s ", arg.name)
#            yield arg
#        for arg in self.obj.params():
#            attribute = 'virtual_attribute' not in arg.flags
#            new_arg = arg.clone(required=True, attribute=attribute, alwaysask=True)
#            new_arg = arg.clone(required=True, attribute=attribute)
#            yield new_arg

    def execute(self, *keys, **options):
        ldap = self.obj.backend

        parent_keys = keys[:-2]

        dn = self.obj.get_dn(*parent_keys, **options)
        entry = self.obj.backend.get_entry(dn, ['objectclass', 'imsConfig'])

        if 'imsconfigobject' not in [o.lower() for o in entry['objectclass']]:
            entry['objectclass'].append('imsConfigObject')

        config = dict()

        if 'imsconfig' in entry:
            config  = dict(s.split('=') for s in list(entry.get('imsconfig', [])))
            entry['configuration'] = config
            entry.pop('imsconfig')

        config[keys[-2]] = keys[-1]

        entry['imsconfig'] = list('='.join((key,val)) for (key,val) in config.items())

        self.obj.backend.update_entry(entry)

        return dict(
            result=True,
            value=pkey_to_value(args[0], keys),
        )

class LDAPConfigUnset(LDAPQuery):
    __doc__ = _('Add new config to entry')

    msg_summary = _('Added new config to "%(value)s"')

    has_output = LDAPQuery.has_output

    takes_params = (
        Str('key',
            cli_name='key',
            label=_('Config key'),
            flags={'virtual_attribute', 'no_create', 'no_update', 'no_search'},
            primary_key=True,
        ),
    )

#    def get_args(self):
#        for arg in super(LDAPConfigUnset, self).get_args():
#            yield arg
#        for arg in self.obj.params():
#            attribute = 'virtual_attribute' not in arg.flags
#            new_arg = arg.clone(required=True, attribute=attribute, alwaysask=True)
#            new_arg = arg.clone(required=True, attribute=attribute)
#            yield new_arg

    def execute(self, *keys, **options):
        ldap = self.obj.backend

        parent_keys = keys[:-1]

        dn = self.obj.get_dn(*parent_keys, **options)
        entry = self.obj.backend.get_entry(dn, ['objectclass', 'imsConfig'])

        if 'imsconfig' in entry:
            config = dict()
            config  = dict(s.split('=') for s in list(entry.get('imsconfig', [])))
            entry['configuration'] = config
            entry.pop('imsconfig')

            if keys[-1] in config:
                config.pop(keys[-1])

            if not config:
                try:
                    del entry['imsconfig']
                except KeyError:
                    pass

                while 'imsconfigobject' in [o.lower() for o in entry['objectclass']]:
                    entry['objectclass'].remove('imsconfigobject')
            else:
                entry['imsconfig'] = list('='.join((key,val)) for (key,val) in config.items())

            self.obj.backend.update_entry(entry)

        return dict(
            result=True,
            value=pkey_to_value(args[0], keys),
        )


class LDAPConfigClear(LDAPQuery):
    __doc__ = _('Clear all config in entry')

    msg_summary = _('Cleared config "%(value)s"')

    def execute(self, *keys, **options):
        ldap = self.obj.backend

        dn = self.obj.get_dn(*keys, **options)

        entry = self.obj.backend.get_entry(dn, ['objectclass', 'imsConfig'])

        try:
            del entry['imsconfig']
        except KeyError:
            pass

        while 'imsconfigobject' in [o.lower() for o in entry['objectclass']]:
            entry['objectclass'].remove('imsconfigobject')

        self.obj.backend.update_entry(entry)

        return dict(
            result=True,
            value=pkey_to_value(args[0], keys),
        )

@register()
class machine_config_set(LDAPConfigSet):
    __doc__ = _('Add new config to machine entry')
    msg_summary = _('Added new config to machine "%(value)s"')

@register()
class machine_config_unset(LDAPConfigUnset):
    __doc__ = _('Remove config variable from machine entry')
    msg_summary = _('Remove config variable from machine entry "%(value)s"')

@register()
class machine_config_clear(LDAPConfigClear):
    __doc__ = _('Clear all config from machine entry')
    msg_summary = _('Cleared all config from machine entry "%(value)s"')


@register()
class machinegroup_config_set(LDAPConfigSet):
    __doc__ = _('Add new config to machine group entry')
    msg_summary = _('Added new config to machine group "%(value)s"')

@register()
class machinegroup_config_unset(LDAPConfigUnset):
    __doc__ = _('Remove config variable from machine group entry')
    msg_summary = _('Remove config variable from machine group entry "%(value)s"')

@register()
class machinegroup_config_clear(LDAPConfigClear):
    __doc__ = _('Clear all config from machine group entry')
    msg_summary = _('Cleared all config from machine group entry "%(value)s"')

@register()
class machinerole_config_set(LDAPConfigSet):
    __doc__ = _('Add new config to machine role entry')
    msg_summary = _('Added new config to machine role "%(value)s"')

@register()
class machinerole_config_unset(LDAPConfigUnset):
    __doc__ = _('Remove config variable from machine role entry')
    msg_summary = _('Remove config variable from machine role entry "%(value)s"')

@register()
class machinerole_config_clear(LDAPConfigClear):
    __doc__ = _('Clear all config from machine role entry')
    msg_summary = _('Cleared all config from machine role entry "%(value)s"')

@register()
class network_config_set(LDAPConfigSet):
    __doc__ = _('Add new config to network entry')
    msg_summary = _('Added new config to network "%(value)s"')

@register()
class network_config_unset(LDAPConfigUnset):
    __doc__ = _('Remove config variable from network entry')
    msg_summary = _('Remove config variable from network entry "%(value)s"')

@register()
class network_config_clear(LDAPConfigClear):
    __doc__ = _('Clear all config from network entry')
    msg_summary = _('Cleared all config from network entry "%(value)s"')

@register()
class networkinterface_config_set(LDAPConfigSet):
    __doc__ = _('Add new config to network interface entry')
    msg_summary = _('Added new config to network interface "%(value)s"')

@register()
class networkinterface_config_unset(LDAPConfigUnset):
    __doc__ = _('Remove config variable from network interface entry')
    msg_summary = _('Remove config variable from network interface entry "%(value)s"')

@register()
class networkinterface_config_clear(LDAPConfigClear):
    __doc__ = _('Clear all config from network interface entry')
    msg_summary = _('Cleared all config from network interface entry "%(value)s"')
