import six

from ipalib import api, errors
from ipalib import Bool, Str, errors, Int, Flag
from ipalib import output
from ipalib.plugable import Registry
from ipaserver.plugins.baseldap import (
    LDAPObject,
    LDAPCreate,
    LDAPRetrieve,
    LDAPDelete,
    LDAPUpdate,
    LDAPSearch,
    LDAPAddMember,
    LDAPRemoveMember)
from ipalib import _, ngettext
from ipapython.dn import DN
from ipaserver.plugins.group import group, group_add, group_mod
from ipaserver.plugins.host import host, host_add
from ipaserver.plugins.user import user, user_add, user_mod, user_show

if six.PY3:
    unicode = str

__doc__ = _("""
Machine group

Group of machines an machine specific data.


EXAMPLES:

 Add a new machines group:
   ipa machinegroup-add --desc='3rd floor lab' mygroup

 Delete a machines group:
   ipa machinegroup-del mygroup

 Modify information about a machines group:
   ipa machinegroup-mod --desc='3rd floor lab' mygroup

 Manage machines group membership, commands:
   ipa machinegroup-add-member --machine=test mygroup

 Manage machines group membership, commands:
   ipa machinegroup-remove-member --machine=test mygroup

 Show information about a machines group:
   ipa machinegroup-show mygroup

""")

def get_complete_machinegroup_member_list(machinegroup):
    result = api.Command['machinegroup_show'](machinegroup)['result']
    direct = list(result.get('member_machine', []))
    indirect = list(result.get('memberindirect_machine', []))
    return direct + indirect


register = Registry()

machinegroup_output_params = (
#    Str(
#        'membermanager_group',
#        label='Membership managed by groups',
#    ),
#    Str(
#        'membermanager_user',
#        label='Membership managed by users',
#    ),
#    Str(
#        'membermanager',
#        label=_('Failed member manager'),
#    ),
)

@register()
class machinegroup(LDAPObject):
    """
    Machine group object
    """
    container_dn = DN(('cn', 'machinegroups'))
    object_name = _('machines group')
    object_name_plural = _('machines groups')
    object_class = ['ipaObject', 'imsMachineGroup' ]
    permission_filter_objectclasses = ["imsMachineGroup"]
    search_attributes = [
        'cn', 'description', 'member', 'memberof'
    ]
    default_attributes = [
        'cn', 'description', 'member', 'memberof', 'memberindirect',
        'memberofindirect'
    ]
    uuid_attribute = 'ipauniqueid'
    allow_rename = True
    attribute_members = {
        'member': ['machine', 'machinegroup'],
        'memberof': ['machinegroup'],
        'memberindirect': ['machine', 'machinegroup'],
        'memberofindirect': ['machinegroup']
    }
    bindable = False

    managed_permissions = {
        'System: Read Machine Groups': {
            'replaces_global_anonymous_aci': True,
            'ipapermbindruletype': 'all',
            'ipapermright': {'read', 'search', 'compare'},
            'ipapermdefaultattr': {
                'cn', 'description',
                'objectclass', 'ipauniqueid'
            },
        },
        'System: Read Machine Groups Membership': {
            'replaces_global_anonymous_aci': True,
            'ipapermbindruletype': 'all',
            'ipapermright': {'read', 'search', 'compare'},
            'ipapermdefaultattr': {
                'member', 'memberof',
            },
        },
        'System: Add Machine Groups': {
            'ipapermright': {'add'},
            'replaces': [
                '(target = "ldap:///cn=*,cn=machinegroups,$SUFFIX")(version 3.0;acl "permission:Add Machine Groups";allow (add) groupdn = "ldap:///cn=Add Machine Groups,cn=permissions,cn=pbac,$SUFFIX";)',
            ],
            'default_privileges': {'Machine Groups Administrators'},
        },
        'System: Modify Machine Groups Membership': {
            'ipapermright': {'write'},
            'ipapermtargetfilter': [
                '(objectclass=imsmachinegroup)',
            ],
            'ipapermdefaultattr': {'member'},
            'replaces': [
                '(targetattr = "member")(target = "ldap:///cn=*,cn=hostgroups,cn=accounts,$SUFFIX")(version 3.0;acl "permission:Modify Machine Groups membership";allow (write) groupdn = "ldap:///cn=Modify Machine Groups  membership,cn=permissions,cn=pbac,$SUFFIX";)',
            ],
            'default_privileges': {'Machine Groups Administrators'},
        },
        'System: Modify Machine Groups': {
            'ipapermright': {'write'},
            'ipapermdefaultattr': {
                'cn', 'description', 'l', 'c',
            },
            'replaces': [
                '(targetattr = "cn || description" )(target = "ldap:///cn=*,cn=machinegroups,$SUFFIX")(version 3.0;acl "permission:Modify Machine Groups";allw (write) groupdn = "ldap:///cn=Modify Machine Groups,cn=permissions,cn=pbac,$SUFFIX";)',

            ],
            'default_privileges': {'Machine Groups Administrators'},
        },
        'System: Remove Machine Groups': {
            'ipapermright': {'delete'},
            'replaces': [
                '(target = "ldap:///cn=*,cn=machinegroups,$SUFFIX")(version 3.0;acl "permission:Remove Machine Groups";allow (delete) groupdn = "ldap:///cn=Remove Machine Groups,cn=permissions,cn=pbac,$SUFFIX";)',
            ],
            'default_privileges': {'Machine Groups Administrators'},
        },
    }

    label = _('Machine Groups')
    label_singular = _('Machine Group')

    takes_params = (
        Str('cn',
            cli_name='groupname',
            label=_('Machine group name'),
            primary_key=True,
            normalizer=lambda value: value.lower(),
        ),
        Str('description?',
            cli_name='desc',
            label=_('Description'),
            doc=_('A description of this machine group'),
        ),
    )


@register()
class machinegroup_add(LDAPCreate):
    __doc__ = _('Add a new machine group.')

    has_output_params = LDAPCreate.has_output_params + machinegroup_output_params

    msg_summary = _('Added machine group "%(value)s"')

    def pre_callback(self, ldap, dn, entry_attrs, attrs_list, *keys, **options):
        assert isinstance(dn, DN)
        try:
            # check duplicity with hostgroups first to provide proper error
            api.Object['machinegroup'].get_dn_if_exists(keys[-1])
            self.obj.handle_duplicate_entry(*keys)
        except errors.NotFound:
            pass

        return dn

@register()
class machinegroup_del(LDAPDelete):
    __doc__ = _('Delete a machine group.')

    msg_summary = _('Deleted machine group "%(value)s"')

@register()
class machinegroup_mod(LDAPUpdate):
    __doc__ = _('Modify information about a machine group.')

    has_output_params = LDAPUpdate.has_output_params + machinegroup_output_params

    msg_summary = _('Modified machine group "%(value)s"')

@register()
class machinegroup_find(LDAPSearch):
    __doc__ = _('Search for machine groups.')

    member_attributes = ['member', 'memberof']

    has_output_params = LDAPSearch.has_output_params + machinegroup_output_params

    msg_summary = ngettext(
        '%(count)d machine group matched', '%(count)d machine groups matched', 0
    )

@register()
class machinegroup_show(LDAPRetrieve):
    __doc__ = _('Display information about a machine group.')
    has_output_params = LDAPRetrieve.has_output_params + machinegroup_output_params


@register()
class machinegroup_add_member(LDAPAddMember):
    __doc__ = _('Add members to machine group.')


@register()
class machinegroup_remove_member(LDAPRemoveMember):
    __doc__ = _('Remove members from machine group.')
