import six

from ipalib import api, errors
from ipalib import Bool, Str, errors, Int, Flag, DNParam
from ipalib import output
from ipalib.plugable import Registry
from ipaserver.plugins.baseldap import (
    LDAPObject,
    LDAPCreate,
    LDAPRetrieve,
    LDAPDelete,
    LDAPUpdate,
    LDAPSearch,
    LDAPAddMember,
    LDAPRemoveMember)
from ipalib import _, ngettext
from ipapython.dn import DN
from ipaserver.plugins.group import group, group_add, group_mod
from ipaserver.plugins.host import host, host_add
from ipaserver.plugins.user import user, user_add, user_mod, user_show

if six.PY3:
    unicode = str

__doc__ = _("""
Machine role


EXAMPLES:

 Add a new machines role:
   ipa machinerole-add --desc='3rd floor lab' myrole

 Delete a machines role:
   ipa machinerole-del myrole

 Modify information about a machines role:
   ipa machinerole-mod --desc='3rd floor lab' myrole

 Manage machines role membership, commands:
   ipa machinerole-add-member --machine=test myrole

 Manage machines role membership, commands:
   ipa machinerole-remove-member --machine=test myrole

 Show information about a machines role:
   ipa machinerole-show myrole

""")

#def get_complete_machinerole_member_list(machinerole):
#    result = api.Command['machinerole_show'](machinerole)['result']
#    direct = list(result.get('member_machine', []))
#    return direct


register = Registry()

machinerole_output_params = (
    Str('imsmachinerolemember_machine?',
        label=_('Role member machine'),
    ),
    Str('imsmachineroleinstalledmember_machine?',
        label=_('Installed member machine'),
    ),
)

@register()
class machinerole(LDAPObject):
    """
    Machine role object
    """
    container_dn = DN(('cn', 'machineroles'))
    object_name = _('machine role')
    object_name_plural = _('machine roles')
    object_class = ['ipaObject', 'imsMachineRole' ]
    permission_filter_objectclasses = ["imsMachineRole"]
    search_attributes = [
        'cn', 'description'
    ]
    default_attributes = [
        'cn', 'description', 'imsmachinerolemember', 'imsmachineroleinstalledmember'
    ]
    uuid_attribute = 'ipauniqueid'
    allow_rename = True
    attribute_members = {
        'imsmachinerolemember': ['machine'],
        'imsmachineroleinstalledmember': ['machine'],
    }

    bindable = False

    managed_permissions = {
        'System: Read Machine Roles': {
            'replaces_global_anonymous_aci': True,
            'ipapermbindruletype': 'all',
            'ipapermright': {'read', 'search', 'compare'},
            'ipapermdefaultattr': {
                'cn', 'description',
                'objectclass', 'ipauniqueid',
                'imsMachineRoleMember', 'imsMachineRoleInstalledMember'
            },
        },
        'System: Read Machine Roles Membership': {
            'replaces_global_anonymous_aci': True,
            'ipapermbindruletype': 'all',
            'ipapermright': {'read', 'search', 'compare'},
            'ipapermdefaultattr': {
                'imsMachineRoleMember', 'imsMachineRoleInstalledMember'
            },
        },
        'System: Add Machine Roles': {
            'ipapermright': {'add'},
            'replaces': [
                '(target = "ldap:///cn=*,cn=machineroles,$SUFFIX")(version 3.0;acl "permission:Add Machine Roles";allow (add) groupdn = "ldap:///cn=Add Machine Roles,cn=permissions,cn=pbac,$SUFFIX";)',
            ],
            'default_privileges': {'Machine Roles Administrators'},
        },
        'System: Modify Machine Roles Membership': {
            'ipapermright': {'write'},
            'ipapermtargetfilter': [
                '(objectclass=imsmachinerole)',
            ],
            'ipapermdefaultattr': {'imsMachineRoleMember'},
            'replaces': [
                '(targetattr = "imsMachineRoleMember")(target = "ldap:///cn=*,cn=hostgroups,cn=accounts,$SUFFIX")(version 3.0;acl "permission:Modify Machine Roles membership";allow (write) groupdn = "ldap:///cn=Modify Machine Roles  membership,cn=permissions,cn=pbac,$SUFFIX";)',
            ],
            'default_privileges': {'Machine Roles Administrators'},
        },
        'System: Modify Machine Roles': {
            'ipapermright': {'write'},
            'ipapermdefaultattr': {
                'cn', 'description', 'imsMachineRoleInstalledMember'
            },
            'replaces': [
                '(targetattr = "cn || description || imsMachineRoleInstalledMember" )(target = "ldap:///cn=*,cn=machineroles,$SUFFIX")(version 3.0;acl "permission:Modify Machine Roles";allw (write) groupdn = "ldap:///cn=Modify Machine Roles,cn=permissions,cn=pbac,$SUFFIX";)',

            ],
            'default_privileges': {'Machine Roles Administrators'},
        },
        'System: Remove Machine Roles': {
            'ipapermright': {'delete'},
            'replaces': [
                '(target = "ldap:///cn=*,cn=machineroles,$SUFFIX")(version 3.0;acl "permission:Remove Machine Roles";allow (delete) groupdn = "ldap:///cn=Remove Machine Roles,cn=permissions,cn=pbac,$SUFFIX";)',
            ],
            'default_privileges': {'Machine Roles Administrators'},
        },
    }

    label = _('Machine Roles')
    label_singular = _('Machine Role')

    takes_params = (
        Str('cn',
            cli_name='rolename',
            label=_('Machine role name'),
            primary_key=True,
            normalizer=lambda value: value.lower(),
        ),
        Str('description?',
            cli_name='desc',
            label=_('Description'),
            doc=_('A description of this machine role'),
        ),
    )


@register()
class machinerole_add(LDAPCreate):
    __doc__ = _('Add a new machine role.')

#    has_output_params = LDAPCreate.has_output_params + machinerole_output_params
    msg_summary = _('Added machine role "%(value)s"')
    def pre_callback(self, ldap, dn, entry_attrs, attrs_list, *keys, **options):
        assert isinstance(dn, DN)
        try:
            # check duplicity with hostgroups first to provide proper error
            api.Object['machinerole'].get_dn_if_exists(keys[-1])
            self.obj.handle_duplicate_entry(*keys)
        except errors.NotFound:
            pass

        return dn

@register()
class machinerole_del(LDAPDelete):
    __doc__ = _('Delete a machine role.')

    msg_summary = _('Deleted machine role "%(value)s"')

@register()
class machinerole_mod(LDAPUpdate):
    __doc__ = _('Modify information about a machine role.')

    has_output_params = LDAPUpdate.has_output_params + machinerole_output_params

    msg_summary = _('Modified machine role "%(value)s"')

@register()
class machinerole_find(LDAPSearch):
    __doc__ = _('Search for machine roles.')

    msg_summary = ngettext(
        '%(count)d machine role matched', '%(count)d machine roles matched', 0
    )

@register()
class machinerole_show(LDAPRetrieve):
    __doc__ = _('Display information about a machine role.')

    has_output_params = LDAPRetrieve.has_output_params + machinerole_output_params


@register()
class machinerole_add_member(LDAPAddMember):
    __doc__ = _('Add members to machine role.')

    member_attributes = ['imsmachinerolemember']

@register()
class machinerole_remove_member(LDAPRemoveMember):
    __doc__ = _('Remove members from machine role.')

    member_attributes = ['imsmachinerolemember']

@register()
class machinerole_add_installed_member(LDAPAddMember):
    __doc__ = _('Add members to machine role.')

    member_attributes = ['imsmachineroleinstalledmember']

@register()
class machinerole_remove_installed_member(LDAPRemoveMember):
    __doc__ = _('Remove members from machine role.')

    member_attributes = ['imsmachineroleinstalledmember']
