import six

from ipalib import api, errors
from ipalib import Bool, Str, errors, Int, Flag
from ipalib import output
from ipalib.plugable import Registry
from ipalib.util import validate_domain_name, normalize_hostname
from ipaserver.plugins.baseldap import (
    LDAPObject,
    LDAPQuery,
    LDAPCreate,
    LDAPRetrieve,
    LDAPDelete,
    LDAPUpdate,
    LDAPSearch,
    LDAPAddMember,
    LDAPRemoveMember,
    LDAPAddAttribute,
    LDAPModAttribute,
    LDAPRemoveAttribute,
    add_missing_object_class,
    pkey_to_value)
from ipalib import _, ngettext
from ipapython.dn import DN
from ipaserver.plugins.group import group, group_add, group_mod
from ipaserver.plugins.host import host, host_add
from ipaserver.plugins.user import user, user_add, user_mod, user_show

if six.PY3:
    unicode = str

__doc__ = _("""
Machine
""") + _("""
A machine represents physical server, virtual machine, container or
any other device which could be provisioned.

EXAMPLES:
 Add a new machine:
   ipa machine-add --location="3rd floor lab"

 Delete a machine:
   ipa machine-del test.example.com

 Modify information about a machine:
   ipa machine-mod --location='de' test.example.com

 Add role to machine
   ipa machine-role-add test.example.com newrole role_state

 Remove role from machine
   ipa machine-role-del test.example.com newrole

 Add machine config variable
   ipa machine-config-add test.example.com 

 Remove machine config variable
   ipa machine-config-remove test.example.com newrole
""")

register = Registry()

machine_output_params = (
    Str('imsmachinerolememberof?',
        label=_('Member of Roles '),
    ),
    Str('imsmachineroleinstalledmemberof?',
        label='Installed roles',
    ),
)

@register()
class machine(LDAPObject):
    """
    Machine object
    """
    container_dn = DN(('cn', 'machines'))
    object_name = _('machine')
    object_name_plural = _('machines')
    object_class = ['ipaobject', 'imsMachine' ]
    permission_filter_objectclasses = ["imsMachine"]
    search_attributes = [
        'fqdn', 'description', 'c', 'l'
    ]
    default_attributes = [
        'fqdn', 'description', 'c', 'l', 'memberof',
        'imsmachinerolememberof', 'imsmachineroleinstalledmemberof'
    ]
    uuid_attribute = 'ipauniqueid'
    attribute_members = {
        'memberof': ['machinegroup'],
        'imsmachinerolememberof': ['machinerole'],
        'imsmachineroleinstalledmemberof': ['machinerole'],
    }

    reverse_members = {
        'member' : ['macinegroup'],
        'imsmachinerolemember' : ['macinerole'],
        'imsmachineinstalledrolemember' : ['macinerole'],
    }

    bindable = False

    relationships = {
        'memberof': ('Member Of', 'in_', 'not_in_'),
        'imsmachinerolememberof': ('Role Member Of', 'in_', 'not_in_'),
        'imsmachineroleinstalledmemberof': ('Installed Role Member Of', 'in_', 'not_in_'),
    }

    managed_permissions = {
        'System: Read Machines': {
            'replaces_global_anonymous_aci': True,
            'ipapermbindruletype': 'all',
            'ipapermright': {'read', 'search', 'compare'},
            'ipapermdefaultattr': {
                'cn', 'description', 'fqdn', 'c', 'l',
                'objectclass', 'ipauniqueid', 'memberof',
                'imsmachinerolememberof', 'imsmachineroleinstalledmemberof'
            },
        },
        'System: Add Machines': {
            'ipapermright': {'add'},
            'replaces': [
                '(target = "ldap:///fqdn=*,cn=machines,$SUFFIX")(version 3.0;acl "permission:Add Machines";allow (add) groupdn = "ldap:///cn=Add Machines,cn=permissions,cn=pbac,$SUFFIX";)',
            ],
            'default_privileges': {'Machine Administrators'},
        },
        'System: Modify Machines': {
            'ipapermright': {'write'},
            'ipapermdefaultattr': {
                'description', 'c', 'l'
            },
            'replaces': [
                '(targetattr = "description || c || l"  )(target = "ldap:///fqdn=*,cn=machines,$SUFFIX")(version 3.0;acl "permission:Modify Machines";allw (write) groupdn = "ldap:///cn=Modify Machines,cn=permissions,cn=pbac,$SUFFIX";)',

            ],
            'default_privileges': {'Machine Administrators'},
        },

        'System: Remove Machines': {
            'ipapermright': {'delete'},
            'replaces': [
                '(target = "ldap:///fqdn=*,cn=machines,$SUFFIX")(version 3.0;acl "permission:Remove Machines";allow (delete) groupdn = "ldap:///cn=Remove Machines,cn=permissions,cn=pbac,$SUFFIX";)',
            ],
            'default_privileges': {'Machine Administrators'},
        },
    }

    label = _('Machines')
    label_singular = _('Machine')

    takes_params = (
        Str('fqdn',
            cli_name='machinename',
            label=_('Machine name'),
            primary_key=True,
            normalizer=normalize_hostname,
        ),
        Str('description?',
            cli_name='desc',
            label=_('Description'),
            doc=_('A description of this machine'),
        ),
        Str('c?',
            cli_name='country',
            label=_('Country'),
            doc=_('Machine contry'),
        ),
        Str('l?',
            cli_name='locality',
            label=_('Locality'),
            doc=_('Machine locality")'),
        )
    )

@register()
class machine_add(LDAPCreate):
    __doc__ = _('Add a new machine.')

    msg_summary = _('Added machine "%(value)s"')

    has_output_params = LDAPCreate.has_output_params + machine_output_params

    def pre_callback(self, ldap, dn, entry_attrs, attrs_list, *keys, **options):
        assert isinstance(dn, DN)
        config = ldap.get_ipa_config()
        if 'ipamaxhostnamelength' in config:
            maxlen = int(config.get('ipamaxhostnamelength')[0])
            if len(keys[-1]) > maxlen:
                raise errors.ValidationError(
                    name=self.obj.primary_key.cli_name,
                    error=_('can be at most %(len)d characters' %
                            dict(len=maxlen))
                )
        if 'locality' in entry_attrs:
            entry_attrs['l'] = entry_attrs['locality']
        entry_attrs['cn'] = keys[-1]
        return dn

@register()
class machine_del(LDAPDelete):
    __doc__ = _('Delete a machine.')

    msg_summary = _('Deleted machine "%(value)s"')

@register()
class machine_mod(LDAPUpdate):
    __doc__ = _('Modify information about a machine.')

    msg_summary = _('Modified machine "%(value)s"')

    takes_options = LDAPUpdate.takes_options

    has_output_params = LDAPUpdate.has_output_params + machine_output_params

    def pre_callback(self, ldap, dn, entry_attrs, attrs_list, *keys, **options):
        assert isinstance(dn, DN)
        fqdn = keys[-1]
        # Once a principal name is set it cannot be changed
        if 'cn' in entry_attrs:
            raise errors.ACIError(info=_('cn is immutable'))
        if 'locality' in entry_attrs:
            entry_attrs['l'] = entry_attrs['locality']

        return dn

    def post_callback(self, ldap, dn, entry_attrs, *keys, **options):
        assert isinstance(dn, DN)
        return dn

@register()
class machine_find(LDAPSearch):
    __doc__ = _('Search for machines.')
    msg_summary = ngettext(
        '%(count)d machine matched', '%(count)d machines matched', 0
    )

    has_output_params = LDAPSearch.has_output_params + machine_output_params

    def pre_callback(self, ldap, filter, attrs_list, base_dn, scope, *args, **options):
        assert isinstance(base_dn, DN)
        if 'locality' in attrs_list:
            attrs_list.remove('locality')
            attrs_list.append('l')
        return (filter.replace('locality', 'l'), base_dn, scope)

@register()
class machine_show(LDAPRetrieve):
    __doc__ = _('Display information about a machine.')

    has_output_params = LDAPRetrieve.has_output_params + machine_output_params

