import six

from ipalib import api, errors
from ipalib import Bool, Str, errors, Int, Flag
from ipalib import output
from ipalib.plugable import Registry
from ipaserver.plugins.baseldap import (
    LDAPObject,
    LDAPQuery,
    LDAPCreate,
    LDAPRetrieve,
    LDAPDelete,
    LDAPUpdate,
    LDAPSearch,
    add_missing_object_class,
    pkey_to_value )
from ipalib import _, ngettext
from ipapython.dn import DN
from ipaserver.plugins.group import group, group_add, group_mod
from ipaserver.plugins.host import host, host_add
from ipaserver.plugins.user import user, user_add, user_mod, user_show

if six.PY3:
    unicode = str

__doc__ = _("""
Network
""") + _("""
Network object.

""") + _("""
EXAMPLES:
 Add a new network:
   ipa network-add --description="My network" test
""") + _("""
 Delete a network:
   ipa network-del test
""") + _("""
 Modify information about a network:
   ipa network-mod --description='Some network' test
""") + _("""
 Find network:
   ipa network-find --description='Some network' 
""")

register = Registry()

network_output_params = (
#    Flag('has_keytab',
#        label=_('Keytab'),
#    ),
#    Str('managedby_host',
#        label='Managed by',
#    ),
)

@register()
class network(LDAPObject):
    """
    Network object
    """
    container_dn = DN(('cn', 'networks'))
    object_name = _('network')
    object_name_plural = _('networks')
    object_class = ['ipaobject', 'imsNetwork' ]
    permission_filter_objectclasses = ["imsNetwork"]
    search_attributes = [
        'imsnetworkname', 'description', 'imsNetworkZone'
    ]
    default_attributes = [
        'imsnetworkname', 'description', 'imsNetworkZone'
    ]
    uuid_attribute = 'ipauniqueid'
    rdn_attribute = 'imsnetworkname'
    allow_rename = True
    managed_permissions = {
        'System: Read Networks': {
            'replaces_global_anonymous_aci': True,
            'ipapermbindruletype': 'all',
            'ipapermright': {'read', 'search', 'compare'},
            'ipapermdefaultattr': {
                'imsnetworkname', 'description', 'imsNetworkZone',
                'objectclass', 'ipauniqueid'
            },
        },
        'System: Add Networks': {
            'ipapermright': {'add'},
            'replaces': [
                '(target = "ldap:///imsnetworkname=*,cn=networks,$SUFFIX")(version 3.0;acl "permission:Add Networks";allow (add) groupdn = "ldap:///cn=Add Networks,cn=permissions,cn=pbac,$SUFFIX";)',
            ],
            'default_privileges': {'Networks Administrators'},
        },
        'System: Modify Networks': {
            'ipapermright': {'write'},
            'ipapermdefaultattr': {
                'imsnetworkname', 'description', 'imsNetworkZone'
            },
            'replaces': [
                '(targetattr = "imsnetworkname || description || imsNetworkZone" )(target = "ldap:///imsnetworkname=*,cn=networks,$SUFFIX")(version 3.0;acl "permission:Modify Networks";allw (write) groupdn = "ldap:///cn=Modify Networks,cn=permissions,cn=pbac,$SUFFIX";)',

            ],
            'default_privileges': {'Networks Administrators'},
        },

        'System: Remove Networks': {
            'ipapermright': {'delete'},
            'replaces': [
                '(target = "ldap:///imsnetworkname=*,cn=networks,$SUFFIX")(version 3.0;acl "permission:Remove Networks";allow (delete) groupdn = "ldap:///cn=Remove Networks,cn=permissions,cn=pbac,$SUFFIX";)',
            ],
            'default_privileges': {'Networks Administrators'},
        },
    }

    label = _('Networks')
    label_singular = _('Network')

    takes_params = (
        Str('imsnetworkname',
            cli_name='networkname',
            label=_('Network name'),
            primary_key=True,
        ),
        Str('description?',
            cli_name='desc',
            label=_('Description'),
            doc=_('A description of this network'),
        ),
    )


@register()
class network_add(LDAPCreate):
    __doc__ = _('Add a new network.')

    msg_summary = _('Added network "%(value)s"')

    has_output_params = LDAPCreate.has_output_params + network_output_params

@register()
class network_del(LDAPDelete):
    __doc__ = _('Delete a network.')

    msg_summary = _('Deleted network "%(value)s"')

@register()
class network_mod(LDAPUpdate):
    __doc__ = _('Modify information about a network.')

    msg_summary = _('Modified network "%(value)s"')

    has_output_params = LDAPUpdate.has_output_params + network_output_params

@register()
class network_find(LDAPSearch):
    __doc__ = _('Search for networks.')

    msg_summary = ngettext(
        '%(count)d network matched', '%(count)d networks matched', 0
    )

@register()
class network_show(LDAPRetrieve):
    __doc__ = _('Display information about a network.')

    has_output_params = LDAPRetrieve.has_output_params + network_output_params








