import six

from ipalib import api, errors
from ipalib import Bool, Str, errors, Int, Flag, DNParam
from ipalib import output
from ipalib.plugable import Registry
from ipaserver.plugins.baseldap import (
    LDAPObject,
    LDAPQuery,
    LDAPCreate,
    LDAPRetrieve,
    LDAPDelete,
    LDAPUpdate,
    LDAPSearch,
    add_missing_object_class,
    pkey_to_value )
from ipalib import _, ngettext
from ipapython.dn import DN

if six.PY3:
    unicode = str

__doc__ = _("""
Network interface

 Add network interface
   ipa networkinterface-add test.example.com eth0

 Set param to network interface
   ipa networkinterface-set test.example.com newrole required

 Show all machines roles and states
   ipa networkinterface-show test.example.com

 Show specific machines roles and states
   ipa networkinterface-show test.example.com somerole
""")

register = Registry()

networkinterface_output_params = (
#    Flag('has_keytab',
#        label=_('Keytab'),
#    ),
#    Str('managedby_host',
#        label='Managed by',
#    ),
)

@register()
class networkinterface(LDAPObject):
    """
    Network interface
    """
    container_dn = DN(('cn', 'machines'))
    parent_object = 'machine'
    object_name = _('network interface')
    object_name_plural = _('network interfaces')
    object_class = ['imsNetworkInterface' ]
    permission_filter_objectclasses = [ 'imsNetworkInterface']
    search_attributes = [
        'imsNetworkInterfaceName', 'description', 'imsNetworkDN'
    ]
    default_attributes = [
        'imsNetworkInterfaceName', 'description', 'imsNetworkDN'
    ]
    managed_permissions = {
        'System: Read Network interfaces': {
            'replaces_global_anonymous_aci': True,
            'ipapermbindruletype': 'all',
            'ipapermright': {'read', 'search', 'compare'},
            'ipapermdefaultattr': {
                'imsNetworkInterfaceName', 'description', 'imsNetworkDN',
                'objectclass'
            },
        },
        'System: Add Network interfaces': {
            'ipapermright': {'add'},
            'replaces': [
                '(target = "ldap:///fqdn=*,cn=machines,$SUFFIX")(version 3.0;acl "permission:Add Network interfaces";allow (add) groupdn = "ldap:///cn=Add Network interfaces,cn=permissions,cn=pbac,$SUFFIX";)',
            ],
            'default_privileges': {'Machine Administrators'},
        },
        'System: Modify Network interfaces': {
            'ipapermright': {'write'},
            'ipapermdefaultattr': {
                'imsNetworkInterfaceName', 'description', 'imsNetworkDN'
            },
            'replaces': [
                '(targetattr = "imsNetworkInterfaceName || description || imsNetworkDN"  )(target = "ldap:///fqdn=*,cn=machines,$SUFFIX")(version 3.0;acl "permission:Modify Network interfaces";allw (write) groupdn = "ldap:///cn=Modify Network interfaces,cn=permissions,cn=pbac,$SUFFIX";)',
            ],
            'default_privileges': {'Machine Administrators'},
        },
        'System: Remove Network interfaces': {
            'ipapermright': {'delete'},
            'replaces': [
                '(target = "ldap:///fqdn=*,cn=machines,$SUFFIX")(version 3.0;acl "permission:Remove Network interfaces";allow (delete) groupdn = "ldap:///cn=Remove Network interfaces,cn=permissions,cn=pbac,$SUFFIX";)',
            ],
            'default_privileges': {'Machine Administrators'},
        },
    }

    label = _('Network interfaces')
    label_singular = _('Network interface')

    takes_params = (
        Str('imsnetworkinterfacename',
            cli_name='interfacename',
            label=_('Interface'),
             doc=_('Network interface name.'),
            primary_key=True,
        ),
        DNParam('imsnetworkdn',
            cli_name='network',
            label=_('Network'),
            doc=_('Network that is connectect to this network interface'),
        ),
        Str('description?',
            cli_name='desc',
            label=_('Description'),
            doc=_('A description of this network interface'),
        ),
    )


@register()
class networkinterface_add(LDAPCreate):
    __doc__ = _('Add a new network interface.')

    msg_summary = _('Added network interface "%(value)s"')

    has_output_params = LDAPCreate.has_output_params + networkinterface_output_params

@register()
class networkinterface_del(LDAPDelete):
    __doc__ = _('Delete a network interface.')

    msg_summary = _('Deleted network interface "%(value)s"')

@register()
class networkinterface_mod(LDAPUpdate):
    __doc__ = _('Modify information about a network interface.')

    msg_summary = _('Modified network interface "%(value)s"')

    has_output_params = LDAPUpdate.has_output_params + networkinterface_output_params

@register()
class networkinterface_find(LDAPSearch):
    __doc__ = _('Search for network interfaces.')
    msg_summary = ngettext(
        '%(count)d network interface matched', '%(count)d network interfaces matched', 0
    )

    has_output_params = LDAPSearch.has_output_params + networkinterface_output_params

@register()
class networkinterface_show(LDAPRetrieve):
    __doc__ = _('Display information about a network interface.')

    has_output_params = LDAPRetrieve.has_output_params + networkinterface_output_params
